
require File.expand_path("../../test_helper", __FILE__)

class Stalking::ProducerTest < Minitest::Test
  def setup
    Beanstalk::Pool.clear!
  end

  def test_enqueue
    Stalking::Producer.new.enqueue "test", { "key" => "value" }, :pri => 100, :delay => 100, :ttr => 100

    assert_equal ['["test",{"key":"value"}]', 100, 100, 100], Beanstalk::Pool.new(["localhost:11300"]).queues["test"].last
  end

  def test_defaults
    Stalking::Producer.new.enqueue "test", "key" => "value"

    assert_equal ['["test",{"key":"value"}]', 65536, 0, 120], Beanstalk::Pool.new(["localhost:11300"]).queues["test"].last
  end

  def test_reconnect
    servers = ["server1", "server2"]

    producer = Stalking::Producer.new(:servers => servers)

    server1 = Beanstalk::Pool.new(["server1"])
    server2 = Beanstalk::Pool.new(["server2"])

    servers.expects(:shuffle).returns(["server1", "server2"]).once

    producer.enqueue "test", "up" => "true"
    assert_equal ['["test",{"up":"true"}]', 65536, 0, 120], server1.queues["test"].last

    server1.down!

    servers.expects(:shuffle).returns(["server1", "server2"]).once

    producer.enqueue "test", "down" => "permanent"
    assert_equal ['["test",{"down":"permanent"}]', 65536, 0, 120], server2.queues["test"].last
  end

  def test_timeout
    servers = ["server1", "server2"]

    producer = Stalking::Producer.new(:servers => servers)

    server1 = Beanstalk::Pool.new(["server1"])
    server2 = Beanstalk::Pool.new(["server2"])

    servers.expects(:shuffle).returns(["server1", "server2"]).once

    producer.enqueue "test", "up" => "true"
    assert_equal ['["test",{"up":"true"}]', 65536, 0, 120], server1.queues["test"].last

    server1.delay = 5

    servers.expects(:shuffle).returns(["server1", "server2"]).once

    producer.enqueue "test", "down" => "delayed"
    assert_equal ['["test",{"down":"delayed"}]', 65536, 0, 120], server2.queues["test"].last
  end

  def test_logger
    logger = TestLogger.new

    producer = Stalking::Producer.new(:logger => logger, :tries => 2)

    Beanstalk::Pool.new(["localhost:11300"]).down!

    producer.enqueue "test", "up" => "true"

    assert_equal ['Enqueue "test" with {"up"=>"true"} and {}', "Beanstalk::NotConnected", "Beanstalk::NotConnected", 'Enqueue "test" with {"up"=>"true"} and {} failed'], logger.entries.collect(&:to_s)
  end
end

